@@ -127,7 +127,7 @@ def nnncomp(text):
   # Each dict represents one token and its attributes
   matcher = Matcher(nlp.vocab)
   # Add with ID, optional callback and pattern(s)
-  pattern = [{"POS": "NOUN"},{"POS": "NOUN","OP":"+"}]
+  pattern = [{"POS": "NOUN"},{"POS": "NOUN","OP":"{1,2}"}] # only macth maxium 3 nouns 15.10.2022
   matcher.add('NA+', [pattern])
   doc = nlp(text)
   # Match by calling the matcher on a Doc object
@@ -440,7 +440,7 @@ def adjNVmatch(text5):
   pattern2 = [{"POS": "ADJ","OP":"+"},{"POS": "NOUN","OP":"+"}]
   matcher.add('A+N', [pattern2])
 
-  pattern3 = [{"POS": "VERB"},{"POS": "NOUN","OP":"+"}]
+  pattern3 = [{"POS": "VERB","OP":"+"},{"POS": "NOUN","OP":"+"}]   # 09.10.2022 {"POS": "VERB"}
   matcher.add('V+N', [pattern3])
 
   pattern4 = [{"POS": "NOUN","OP":"+"},{"LOWER": "be","OP":"+"},{"POS": "VERB"}]
@@ -461,19 +461,48 @@ def adjNVmatch(text5):
   return adjNverbs2 
 
 
-def emo(text):
+#text = "We 😊 recommend 😊and book your next stay at the Arctic Light 😊Hotel😊"
 
-  emojis = []
-  words = []
-  for x in text.split():
-    if emoji.is_emoji(x):
-      x1 = emoji.demojize(x)
-      emojis.append(x1)
+emoji_pattern = re.compile("["
+        u"\U0001F600-\U0001F64F"  # emoticons
+        u"\U0001F300-\U0001F5FF"  # symbols & pictographs
+        u"\U0001F680-\U0001F6FF"  # transport & map symbols
+        u"\U0001F1E0-\U0001F1FF"  # flags (iOS)
+                           "]+", flags=re.UNICODE)   # this patter is not full, use emoji module instead 
+
+def emo(text, emojiINtext = False):
+    """
+    :smiling_face_with_smiling_eyes: : will be replace by , but _ stay 
+    """
+    #text = 'I love food 😊, ok👍👍 bad😱😱 or 🙂great🙂 🙂 💀 😂? '
+    #text = 'how good food ❤️ strong❤️😊 recommendation.'
+    emojis = []   
+    for emoD in emoji.emoji_list(text):
+        emo = emoD['emoji']
+        emojis.append(emo)
+
+    if len(emojis)>0:
+        new_text = text
+        for e in emojis:
+            new_text = new_text.replace(e,' , ')
     else:
-      x2 = x
-      words.append(x2)
-  new_text = " ".join(words)
-  return new_text, emojis
+        new_text = text 
+        
+    text2 = text
+    for emo in emojis:
+        emo_text = emoji.demojize(emo)
+        #emo_text = emo_text.split('_')
+        #emo_text ="".join(emo_text)
+        emo_text = re.sub(r'[^\w\s]', '', emo_text)
+        mapnoun(emo_text)
+        text2 = text2.replace(emo," , " + emo_text +" , ")
+
+    if emojiINtext:
+        return text2,emojis
+    else:
+        return new_text, emojis 
+
+
   
 def vaderAV():
   vader_text = pd.read_csv("https://raw.githubusercontent.com/pengKiina/KeypartX/main/vader_lexicon.csv")
@@ -512,9 +541,10 @@ def vaderAV():
 
 # new for back and again
 
-def abverb(abtext):
+def abverb_old(abtext,myBA = False):
     """
-    again, back # visit, come, go, travel, be 
+    again, back # visit, come, go, travel, be and others verbs 
+    myBA:  my own abverbs ['visit', 'come','go', 'travel', 'be']
     """
     #abtext = "We notlove2vvv it again to Finland, I came back to Finland"
     #abtext ="I come to Finland again"
@@ -561,16 +591,222 @@ def abverb(abtext):
                         ab1 =  token.text
                         ab = ab1
             #print(abV, abV2,ab)
-            if len(abV)>1:
-                #print(abd)
-                abd1 = abd.text.replace('2vvv',"")
-                #print(abd1)
-                ab_verb = abV2 +ab +'2vvv'
-                mapverb(ab_verb)  
-                abd2 =  re.sub(r'\b' +abV + r'\b',ab_verb, abd1)  # replace whole word 
-                #print(abd2)
-                abtext = abtext.replace(abd.text,abd2)
-                ab_verbs.append(ab_verb)
+            if myBA:  # my own abverbs 
+                if len(abV)>1 and any([x in abV2 for x in myBA ]): # changed len(abV)>1 and abV2 in myBA: 09.10.2022
+                    #print(abd)
+                    abd1 = abd.text.replace('2vvv',"")
+                    #print(abd1)
+                    ab_verb = abV2 +ab +'2vvv'
+                    mapverb(ab_verb)  
+                    abd2 =  re.sub(r'\b' +abV + r'\b',ab_verb, abd1)  # replace whole word 
+                    #print(abd2)
+                    abtext = abtext.replace(abd.text,abd2)
+                    ab_verbs.append(ab_verb)
+                    
+            else:
+                if len(abV)>1:
+                    #print(abd)
+                    abd1 = abd.text.replace('2vvv',"")
+                    #print(abd1)
+                    ab_verb = abV2 +ab +'2vvv'
+                    mapverb(ab_verb)  
+                    abd2 =  re.sub(r'\b' +abV + r'\b',ab_verb, abd1)  # replace whole word 
+                    #print(abd2)
+                    abtext = abtext.replace(abd.text,abd2)
+                    ab_verbs.append(ab_verb)                  
+            
         return abtext, ab_verbs 
     else:
-        return abtext, []
\ No newline at end of file
+        return abtext, []
+      
+      
+def abverb_old2(abtext,myBA = False,myBAverb = False):
+    """
+    myBA: ['back','again']
+    myBAverb:  my own abverbs ['visit', 'come','go', 'travel', 'be']
+    """
+    matcher1 = Matcher(nlp.vocab)
+    if myBA:
+        for myba in myBA:
+            #abtext = "We notlove2vvv it again to Finland, I came back to Finland"
+            #abtext ="I come to Finland again"
+            pattern1 = [{"POS": 'VERB'}, {'IS_ASCII': True, 'OP': '{0,3}'},{"LOWER": myba}] #"OP":"+" 1 or more * 0 or more  # ADP: 'from' ,'to'
+            matcher1.add('verb + back', [pattern1])
+            pattern2 = [{"LEMMA": {"IN": ["are", "am","been","is","was",'be','were']}}, {'IS_ASCII': True, 'OP': '{0,3}'},{"LOWER": myba}]#"OP":"+" 1 or more * 0 or more
+            matcher1.add('be + again', [pattern2])
+    else:
+        pattern1 = [{"POS": 'VERB'}, {'IS_ASCII': True, 'OP': '{0,3}'},{"LOWER": 'back'}] #"OP":"+" 1 or more * 0 or more  # ADP: 'from' ,'to'
+        matcher1.add('verb + back', [pattern1])
+        pattern2 = [{"POS": 'VERB'}, {'IS_ASCII': True, 'OP': '{0,3}'},{"LOWER": 'again'}] #"OP":"+" 1 or more * 0 or more  # ADP: 'from' ,'to'
+        matcher1.add('verb + back', [pattern2])
+        pattern3 = [{"LEMMA": {"IN": ["are", "am","been","is","was",'be','were']}}, {'IS_ASCII': True, 'OP': '{0,3}'},{"LOWER": 'back'}]#"OP":"+" 1 or more * 0 or more
+        matcher1.add('be + again', [pattern3])
+
+
+    doc = nlp(abtext)
+    
+    matches = matcher1(doc)
+    spans =[]
+    for match_id, start, end in matches:
+      # Get the matched span by slicing the Doc
+      span = doc[start:end]
+      #print(span.text)
+      if len(span.text.split())>1: #not,not like
+        spans.append(span)
+
+    ab_doc = spacy.util.filter_spans(spans) # only get the longest span
+    #print('ab_doc' , ab_doc)
+    if len(ab_doc)>0:
+        ab_verbs = []
+        for abd in ab_doc:
+
+            abV =""
+            abV2=""
+            ab =""
+            for token in abd:
+                #print(token)
+                if token.pos_ == "VERB" or token.lemma_ =="be":
+                    abv = token.text
+                    abv2 = token.lemma_
+                    abV = abv.replace('2vvv','')
+                    abV2= abv2.replace('2vvv','')
+                   # print('abV',abV)
+                else:
+                    if token.text == "back":
+                        ab1 = token.text
+                        ab = ab1
+                    elif token.text == "again":
+                        ab1 =  token.text
+                        ab = ab1
+            #print(abV, abV2,ab)
+            if myBAverb:  # my own abverbs 
+                #print('yes')
+                if len(abV)>1 and any([x in abV2 for x in myBAverb]): # changed len(abV)>1 and abV2 in myBA: 09.10.2022
+                    #print(abd)
+                    abd1 = abd.text.replace('2vvv',"")
+                    #print(abd1)
+                    ab_verb = abV2 +ab +'2vvv'
+                    mapverb(ab_verb)  
+                    abd2 =  re.sub(r'\b' +abV + r'\b',ab_verb, abd1)  # replace whole word 
+                    #print(abd2)
+                    abtext = abtext.replace(abd.text,abd2)
+                    ab_verbs.append(ab_verb)
+                    
+            else:
+                if len(abV)>1:
+                    #print(abd)
+                    abd1 = abd.text.replace('2vvv',"")
+                    #print(abd1)
+                    ab_verb = abV2 +ab +'2vvv'
+                    mapverb(ab_verb)  
+                    abd2 =  re.sub(r'\b' +abV + r'\b',ab_verb, abd1)  # replace whole word 
+                    #print(abd2)
+                    abtext = abtext.replace(abd.text,abd2)
+                    ab_verbs.append(ab_verb)                  
+            
+        return abtext, ab_verbs 
+    else:
+        return abtext, []
+      
+      
+def abverb(abtext,myBA = False,myBAverb = False):
+    """
+    myBA: ['back','again']
+    myBAverb:  my own abverbs ['visit', 'come','go', 'travel', 'be']
+    """
+    matcher1 = Matcher(nlp.vocab)
+    if myBA:
+        for myba in myBA:
+            #abtext = "We notlove2vvv it again to Finland, I came back to Finland"
+            #abtext ="I come to Finland again"
+            pattern1 = [{"POS": 'VERB'}, {'IS_ASCII': True, 'OP': '{0,3}'},{"LOWER": myba}] #"OP":"+" 1 or more * 0 or more  # ADP: 'from' ,'to'
+            matcher1.add('verb + back', [pattern1])
+            pattern2 = [{"LEMMA": {"IN": ["are", "am","been","is","was",'be','were']}}, {'IS_ASCII': True, 'OP': '{0,3}'},{"LOWER": myba}]#"OP":"+" 1 or more * 0 or more
+            matcher1.add('be + again', [pattern2])
+    else:
+        pattern1 = [{"POS": 'VERB'}, {'IS_ASCII': True, 'OP': '{0,3}'},{"LOWER": 'back'}] #"OP":"+" 1 or more * 0 or more  # ADP: 'from' ,'to'
+        matcher1.add('verb + back', [pattern1])
+        pattern2 = [{"POS": 'VERB'}, {'IS_ASCII': True, 'OP': '{0,3}'},{"LOWER": 'again'}] #"OP":"+" 1 or more * 0 or more  # ADP: 'from' ,'to'
+        matcher1.add('verb + back', [pattern2])
+        pattern3 = [{"LEMMA": {"IN": ["are", "am","been","is","was",'be','were']}}, {'IS_ASCII': True, 'OP': '{0,3}'},{"LOWER": 'back'}]#"OP":"+" 1 or more * 0 or more
+        matcher1.add('be + again', [pattern3])
+
+
+    doc = nlp(abtext)
+    
+    matches = matcher1(doc)
+    spans =[]
+    for match_id, start, end in matches:
+      # Get the matched span by slicing the Doc
+      span = doc[start:end]
+      #print(span.text)
+      if len(span.text.split())>1: #not,not like
+        spans.append(span)
+
+    ab_doc = spacy.util.filter_spans(spans) # only get the longest span
+    #print('ab_doc' , ab_doc)
+    if len(ab_doc)>0:
+        ab_verbs = []
+        for abd in ab_doc:
+
+            abV =""
+            abV2=""
+            ab =""
+            for token in abd:
+                #print(token)
+                if token.pos_ == "VERB" or token.lemma_ =="be":
+                    abv = token.text
+                    abv2 = token.lemma_
+                    abV = abv.replace('2vvv','')
+                    abV2= abv2.replace('2vvv','')
+                   # print('abV',abV)
+                else:
+                    try:
+                        if token.text == "back" or token.text in myBA:
+                            ab1 = token.text
+                            ab = ab1
+                        elif token.text == "again" or token.text in myBA:
+                            ab1 =  token.text
+                            ab = ab1
+                    except:
+                        if token.text == "back":
+                            ab1 = token.text
+                            ab = ab1
+                        elif token.text == "again":
+                            ab1 =  token.text
+                            ab = ab1
+            #print(abV, abV2,ab)  # visited, visit, back 
+            if myBAverb:  # my own abverbs 
+                #print('yes')
+                if len(abV)>1 and any([x in abV2 for x in myBAverb]): # changed len(abV)>1 and abV2 in myBA: 09.10.2022
+                    #print(abd)
+                    abd1 = abd.text.replace('2vvv',"")
+                    #print(abd1)
+                    ab_verb = abV2 +ab +'2vvv'
+                    mapverb(ab_verb)  
+                    abd2 =  re.sub(r'\b' +abV + r'\b',ab_verb, abd1)  # replace whole word 
+                    #print(abd2)
+                    abd2 = re.sub(r'\b' +ab + r'\b',"", abd2) 
+                    abtext = abtext.replace(abd.text,abd2)
+                    ab_verbs.append(ab_verb)
+                    
+            else:
+                if len(abV)>1:
+                    #print(abd)
+                    abd1 = abd.text.replace('2vvv',"")
+                    #print('abd' , abd)
+                    ab_verb = abV2 +ab +'2vvv'
+                    mapverb(ab_verb)  
+                    abd2 =  re.sub(r'\b' +abV + r'\b',ab_verb, abd1)  # replace whole word 
+                    #print('abd2', abd2)
+                    abd2 = re.sub(r'\b' +ab + r'\b',"", abd2) 
+                    abtext = abtext.replace(abd.text,abd2)
+                    #print('abtext1',abtext)
+                    ab_verbs.append(ab_verb)                  
+            
+        return abtext, ab_verbs 
+    else:
+        return abtext, []
+      
+if __name__ == "__main__":
+  print('ok')
\ No newline at end of file
