@@ -18,14 +18,14 @@ import networkx as nx
 
 
 opi_verbs = ['enjoy','love','like','adore','avoid','revisit','desire','dislike','hate','wish','hope','appreciate','value','recommend','unrecommend','astonish','impress','please','satisfy','unsatisfy','surprise','mean','mind','overjoy', 'despair', 'disgust', 'attract', 'encourage', 'favour','favor', 'forget', 'remember','disappoint', 'discourage','fascinate',
-            'dissatisfy', 'disinterest', 'interest', 'complain', 'compliment', 'suffer' , 'trust', 'distrust' 'praise','content','miss']
+            'dissatisfy', 'disinterest', 'interest', 'complain', 'compliment', 'suffer' , 'trust', 'distrust', 'praise','content','miss']
 
 neg_words = ['hardly', 'scarcely','barely','no','not','none','neither','nor','never']
 #keep_words2 = keep_words +opi_verbs + neg_words
 #textneg = "I do not like you never know it be not really pretty, it means no money no girl, I hardly learn in the never nice school"
 #text =""" I don't like this High-quality hotel, /the\ restaraunt food is so ugly and expensive, United states of America next time we will go to New York, "good-good"  the place called "long islands",but I never hate it, it was not pretty,so I don't recommend it."""
 
-nounDEL = ['thing', 'things','issue', 'issues', 'part', 'parts','lot','lots', 'year', 'years', 'week', 'weeks', 'month','months', 'hour','hours', 'date', 'minute',  'minutes', 'one','ones']
+nounDEL = ['thing', 'things','issue', 'issues', 'part', 'parts','lot','lots', 'year', 'years', 'week', 'weeks', 'month','months', 'hour','hours', 'date', 'minute',  'minutes', 'one','ones','bit','bits','addition','variety']
 adjDEL =["same","similar","different","another","other","half", "some", "several", "much", "all", "many", "whole","enough", "little", "sufficient", "most","least","few", "own","very","this", "that", "these", "those","each", "every","any","less","more","only",'next','previous','least','first','second','third','able','last','such','main','really','late','early','possible','due',] # number are deleted separtely 
 #timedate = timedate0 + non_opi_adjs
 #timedate.remove('pengy')
@@ -56,7 +56,7 @@ def negADJ(neg_words,adjDEL):
 
 
 
-def text2edges(text,verbose = False, verbose_text= False,include_otherEdge = False, only_otherNouns = False,only_opiAV = True, opi_verbs = opi_verbs ,neg_words = neg_words, nounDEL =nounDEL, adjDEL = adjDEL,nncompound = False,facility = False,faciNN = False, emojis = True, check_spell = True,back_again = True):
+def text2edges(text,verbose = False, verbose_text= False,include_otherEdge = False, only_otherNouns = False,only_opiAV = True, opi_verbs = opi_verbs ,neg_words = neg_words, nounDEL =nounDEL, adjDEL = adjDEL,nncompound = False,facility = False,faciNN = False, emojis = True,emojiINtext = False, check_spell = True,back_again = True,myBA = False,myBAverb = False):
   """
   include_otherEdge: greedy edges of the unmatched adj, verb, noun 
   only_otherNouns: does not have adjverb in other2N_edges other than av2n, the other2N_edeges are greedy to mach the rest adj, verb, noun instead of considering match pattern
@@ -64,6 +64,7 @@ def text2edges(text,verbose = False, verbose_text= False,include_otherEdge = Fal
   adjDEL : adjective to be removed 
   facility: additional noun for the missing in the include_otherEdge class, e.g. I recommend! So pretty! 
   faciNN: if True add facility noun to all NN edges, otherwise facility only add to AV2N in the otherEdges other than facilty-N in the matched AV2N to form greedy NN 
+  myBA: my own abverbs ['visit', 'come','go', 'travel', 'be'] + 'back', 'again' 
   """
   non_opi_adjs = negADJ(neg_words,adjDEL)
   
@@ -74,9 +75,9 @@ def text2edges(text,verbose = False, verbose_text= False,include_otherEdge = Fal
   # text2 = re.sub(r'\b' + token.text + r'\b'," ", text2)
   #line1 = re.sub(' +', ' ', line1).strip() #remove extra space whitespace 
   if emojis:
-    text, emojis1 = emo(text)
+    text, emojis1 = emo(text, emojiINtext = emojiINtext)
   else:
-    text = text
+    text, _= emo(text)
     emojis1 =[]
   
   ## --- check spelling ---##
@@ -173,7 +174,7 @@ def text2edges(text,verbose = False, verbose_text= False,include_otherEdge = Fal
     except:
       text2 = text2c
       if verbose_text:
-        print('text2: ', text2)
+        print('text2 NO_correference: ', text2)
     #text2 = lemma_en(text2)
 
 
@@ -233,8 +234,12 @@ def text2edges(text,verbose = False, verbose_text= False,include_otherEdge = Fal
 
   ## ---- back , again verb ---
   if back_again:
-    text3,_ = abverb(text3)
-
+    text3,ba_verbs = abverb(text3,myBA = myBA,myBAverb = myBAverb)
+    bavbs = [] 
+    for vb in ba_verbs:
+      vb1 = vb.replace('2vvv','')
+      bavbs.append(vb1)
+    opi_verbs = opi_verbs +bavbs  
 
 
   #5. except compounds, other normal words into ADJ, VERB, NOUN 
@@ -257,7 +262,7 @@ def text2edges(text,verbose = False, verbose_text= False,include_otherEdge = Fal
 
 
   all_new_comps1 = [x.lower() for x in all_new_comps]  # Tripadvisor entity will become tripadvisornnn after text2 lemma_en 
-  all_new_comps1 = all_new_comps1 + all_new_comps +['be']
+  all_new_comps1 = all_new_comps1 + all_new_comps +['be'] +ba_verbs
   for token in nlp(text4):
     if token.text not in all_new_comps1:
 
@@ -359,7 +364,7 @@ def text2edges(text,verbose = False, verbose_text= False,include_otherEdge = Fal
   text5 = nonAcomaA(text5)
 
   adjNverbs2 = adjNVmatch(text5)
-  adjNverbs2 
+  #adjNverbs2 
 
   ## --- 1 adj verb noun edges ---
   adjV2N_edges, nn_edges = av2Nedge(adjNverbs2)
@@ -433,12 +438,12 @@ def text2edges(text,verbose = False, verbose_text= False,include_otherEdge = Fal
 
   ##  --- all edges --- ## 
   if include_otherEdge:
-    if only_otherNouns: # does not have adjverb in other2N_edges other than av2n, the other2N_edeges are greedy to mach the rest adj, verb, noun instead of considering match pattern
+    if only_otherNouns: # does not have other2N_edges but nn edges. does not have adjverb in other2N_edges other than av2n, the other2N_edeges are greedy to mach the rest adj, verb, noun instead of considering match pattern
       all_adjV2N_edges = adjV2N_edges
       allNNedges = allNN_edges0
     else:
       all_adjV2N_edges = adjV2N_edges+other2N_edges
-      allNNedges = allNN_edges0
+      allNNedges = allNN_edges0 
   
   else:
     all_adjV2N_edges = adjV2N_edges
@@ -497,7 +502,7 @@ def targList(mylist):
 
 #from autocorrect import Speller
 #from collections import defaultdict
-def target2edges(text,verbose = False, verbose_text= False,include_otherEdge = False, only_otherNouns = False,opi_verbs = opi_verbs ,neg_words =neg_words,adjDEL = adjDEL,nounDEL = nounDEL,only_opiAV = True,nncompound = True, emojis = True,check_spell = True,back_again = True):
+def target2edges(text,verbose = False, verbose_text= False,include_otherEdge = False, only_otherNouns = False,opi_verbs = opi_verbs ,neg_words =neg_words,adjDEL = adjDEL,nounDEL = nounDEL,only_opiAV = True,nncompound = True, emojis = True,emojiINtext = False,  check_spell = True,back_again = True, myBA =False,myBAverb = False):
   non_opi_adjs = negADJ(neg_words,adjDEL)
   # 1. check spelling 
 
@@ -505,9 +510,9 @@ def target2edges(text,verbose = False, verbose_text= False,include_otherEdge = F
   # text2 = re.sub(r'\b' + token.text + r'\b'," ", text2)
   #line1 = re.sub(' +', ' ', line1).strip() #remove extra space whitespace 
   if emojis:
-    text, emojis2 = emo(text)
+    text, emojis2 = emo(text,emojiINtext = emojiINtext)
   else:
-    text = text
+    text, emojis1 = emo(text)
     emojis2 =[]
 
 
@@ -659,7 +664,14 @@ def target2edges(text,verbose = False, verbose_text= False,include_otherEdge = F
  
   ## ---- back , again verb ---
   if back_again:
-    text3,_ = abverb(text3)
+    text3,ba_verbs = abverb(text3,myBA = myBA,myBAverb = myBAverb)
+    bavbs = [] 
+    for vb in ba_verbs:
+      vb1 = vb.replace('2vvv','')
+      bavbs.append(vb1)
+    opi_verbs = opi_verbs +bavbs  
+
+
 
   #5. ecept compounds, other normal words into ADJ, VERB, NOUN 
 
@@ -681,7 +693,7 @@ def target2edges(text,verbose = False, verbose_text= False,include_otherEdge = F
 
 
   all_new_comps1 = [x.lower() for x in all_new_comps]  # Tripadvisor entity will become tripadvisornnn after text2 lemma_en 
-  all_new_comps1 = all_new_comps1 + all_new_comps +['be']
+  all_new_comps1 = all_new_comps1 + all_new_comps +['be'] +ba_verbs
   for token in nlp(text4):
     if token.text not in all_new_comps1:
 
